searchData={"items":[{"type":"module","title":"Kindling","doc":"Kindling is a library for working with [HL7 FHIR](https://hl7.org/fhir/) APIs. It can generate\neach FHIR resource as an `Ecto.Schema`. It also contains a client for working with the FHIR REST\nAPI.","ref":"Kindling.html"},{"type":"module","title":"Installation - Kindling","doc":"Kindling can be installed by adding `kindling` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kindling, \"~> 1.0.0\"}\n  ]\nend\n```\n\nYou should then configure your root resources, which are the FHIR resources that your application\nuses:\n\n```elixir\nconfig :kindling, root_resources: [\"Bundle\", \"Patient\", \"Encounter\"]\n```\n\nWhen you generate resource schemas, Kindling will generate these schemas, plus any that they reference (recursively).","ref":"Kindling.html#module-installation"},{"type":"module","title":"Generating Resource Schemas - Kindling","doc":"[mix kindling.generate_schemas](`Mix.Tasks.Kindling.GenerateSchemas`) will generate Elixir source files for the resource schemas under a namespace module under the `lib/` directory. It takes two arguments: the name of the namespace module, and a FHIR version.\n\nExample:\n\n```sh\nmix kindling.generate_schemas FHIR R4\n```","ref":"Kindling.html#module-generating-resource-schemas"},{"type":"module","title":"Example Schema Module - Kindling","doc":"The generated schema modules are normal `Ecto.Schema`s. Here's an example of a Patient resource schema:\n\n```elixir\ndefmodule FHIR.R4.Patient do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  @fields [\n    :active,\n    :multiple_birth_boolean,\n    :language,\n    :implicit_rules,\n    :birth_date,\n    :multiple_birth_integer,\n    :id,\n    :deceased_boolean,\n    :gender,\n    :deceased_date_time\n  ]\n  @required_fields []\n\n  @primary_key {:id, :binary_id, autogenerate: true}\n  schema \"patient\" do\n    # Constants\n    field(:resource_type, :string, virtual: true, default: \"Patient\")\n\n    # Fields\n    field(:active, :boolean)\n    field(:multiple_birth_boolean, :boolean)\n    field(:language, :string)\n    field(:implicit_rules, :string)\n    field(:birth_date, :date)\n    field(:multiple_birth_integer, :integer)\n    field(:deceased_boolean, :boolean)\n    field(:deceased_date_time, :string)\n\n    # Enum\n    field(:gender, Ecto.Enum, values: [:male, :female, :other, :unknown])\n\n    # Embed One\n    embeds_one(:marital_status, FHIR.R4.CodeableConcept)\n    embeds_one(:managing_organization, FHIR.R4.Reference)\n    embeds_one(:text, FHIR.R4.Narrative)\n    embeds_one(:meta, FHIR.R4.Meta)\n\n    # Embed Many\n    embeds_many(:photo, FHIR.R4.Attachment)\n    embeds_many(:communication, FHIR.R4.Patient.Communication)\n    embeds_many(:name, FHIR.R4.HumanName)\n    embeds_many(:extension, FHIR.R4.Extension)\n    embeds_many(:telecom, FHIR.R4.ContactPoint)\n    embeds_many(:contained, FHIR.R4.ResourceList)\n    embeds_many(:link, FHIR.R4.Patient.Link)\n    embeds_many(:contact, FHIR.R4.Patient.Contact)\n    embeds_many(:modifier_extension, FHIR.R4.Extension)\n    embeds_many(:identifier, FHIR.R4.Identifier)\n    embeds_many(:general_practitioner, FHIR.R4.Reference)\n    embeds_many(:address, FHIR.R4.Address)\n  end\n\n  def version, do: FHIR.R4\n  def path, do: \"/Patient\"\n\n  def base_changeset(data \\\\ %__MODULE__{}, attrs) do\n    data\n    |> cast(attrs, @fields)\n    |> validate_required(@required_fields)\n  end\nend\n```","ref":"Kindling.html#module-example-schema-module"},{"type":"module","title":"API Client - Kindling","doc":"Kindling also includes a [FHIR REST API client](`Kindling.Client`) that can be used to request resources\nfrom a FHIR server. The client will automatically convert the results to resource schema structs:\n\n```elixir\n# Use the public FHIR test server\nclient = %{\n  base_url: \"http://hapi.fhir.org/baseR4\",\n  auth_mode: :open\n}\n\npatient_id = \"593166\"\n\nKindling.Client.read(client, FHIR.R4.Patient, patient_id)\n```\n\nResults in a struct something like this:\n\n```elixir\n%FHIR.R4.Patient{\n  __meta__: #Ecto.Schema.Metadata<:built, \"patient\">,\n  id: \"593166\",\n  resource_type: \"Patient\",\n  active: nil,\n  multiple_birth_boolean: nil,\n  language: nil,\n  implicit_rules: nil,\n  birth_date: ~D[2000-10-31],\n  multiple_birth_integer: nil,\n  deceased_boolean: nil,\n  deceased_date_time: nil,\n  gender: :female,\n  marital_status: nil,\n  managing_organization: nil,\n  text: %FHIR.R4.Narrative{\n    id: nil,\n    div: \"  Sabrina  SPELLMAN       \",\n    status: :generated,\n    extension: []\n  },\n  meta: %FHIR.R4.Meta{\n    id: nil,\n    last_updated: {:error, :invalid_format},\n    source: \"#NPQrzINFNuDwuDgM\",\n    version_id: \"1\",\n    profile: nil,\n    extension: [],\n    security: [],\n    tag: []\n  },\n  photo: [],\n  communication: [],\n  name: [\n    %FHIR.R4.HumanName{\n      id: nil,\n      family: \"Spellman\",\n      text: nil,\n      given: [\"Sabrina\"],\n      prefix: nil,\n      suffix: nil,\n      use: nil,\n      period: nil,\n      extension: []\n    }\n  ],\n  extension: [],\n  telecom: [\n    %FHIR.R4.ContactPoint{\n      id: nil,\n      rank: nil,\n      value: \"1(845)443-7666\",\n      system: :phone,\n      use: :home,\n      period: nil,\n      extension: []\n    }\n  ],\n  contained: [],\n  link: [],\n  contact: [],\n  modifier_extension: [],\n  identifier: [],\n  general_practitioner: [],\n  address: [\n    %FHIR.R4.Address{\n      id: nil,\n      city: \"Greendale\",\n      country: \"United States\",\n      district: nil,\n      postal_code: \"11199\",\n      state: \"New York\",\n      text: nil,\n      line: [\"1138 Decario Lane\"],\n      type: nil,\n      use: nil,\n      period: nil,\n      extension: []\n    }\n  ]\n}\n```","ref":"Kindling.html#module-api-client"},{"type":"module","title":"Additional Resources - Kindling","doc":"The docs can be found at  .","ref":"Kindling.html#module-additional-resources"},{"type":"module","title":"Kindling.Client","doc":"FHIR API client.","ref":"Kindling.Client.html"},{"type":"function","title":"Kindling.Client.convert_response/2","doc":"","ref":"Kindling.Client.html#convert_response/2"},{"type":"function","title":"Kindling.Client.create/5","doc":"","ref":"Kindling.Client.html#create/5"},{"type":"function","title":"Kindling.Client.delete/5","doc":"","ref":"Kindling.Client.html#delete/5"},{"type":"function","title":"Kindling.Client.read/5","doc":"Make a read REST request against a FHIR API server, for a resource of type `resource_module`\nand the id `id`. Returns a schema struct of the results, or an error.\n\n`opts`:\n- `headers`: additional HTTP request headers to send with the request, as a list of {key, value}\n    pairs.","ref":"Kindling.Client.html#read/5"},{"type":"function","title":"Kindling.Client.search/5","doc":"Make a search REST request against a FHIR API server, for a resource of type `resource_module`\nusing the search params `params`.\n\nReturns a schema struct of the results (usually a FHIR bundle), or an error.\n\n`opts`:\n- `headers`: additional HTTP request headers to send with the request, as a list of {key, value}\n    pairs.","ref":"Kindling.Client.html#search/5"},{"type":"function","title":"Kindling.Client.update/6","doc":"","ref":"Kindling.Client.html#update/6"},{"type":"type","title":"Kindling.Client.access_token/0","doc":"The FHIR access token. Used when :auth_mode is :bearer or :basic. Ignored when :auth_mode is :open.","ref":"Kindling.Client.html#t:access_token/0"},{"type":"type","title":"Kindling.Client.fhir_server_base_url/0","doc":"The base URL of the FHIR server. Often (but not always), includes the FHIR version in the URL path.","ref":"Kindling.Client.html#t:fhir_server_base_url/0"},{"type":"type","title":"Kindling.Client.resource_id/0","doc":"A string that is the ID of a FHIR resource.","ref":"Kindling.Client.html#t:resource_id/0"},{"type":"type","title":"Kindling.Client.t/0","doc":"Configuration of a FHIR client.","ref":"Kindling.Client.html#t:t/0"},{"type":"module","title":"Kindling.Client.Req","doc":"","ref":"Kindling.Client.Req.html"},{"type":"function","title":"Kindling.Client.Req.delete/2","doc":"","ref":"Kindling.Client.Req.html#delete/2"},{"type":"function","title":"Kindling.Client.Req.get/2","doc":"","ref":"Kindling.Client.Req.html#get/2"},{"type":"function","title":"Kindling.Client.Req.post/2","doc":"","ref":"Kindling.Client.Req.html#post/2"},{"type":"function","title":"Kindling.Client.Req.put/2","doc":"","ref":"Kindling.Client.Req.html#put/2"},{"type":"module","title":"Kindling.Config","doc":"Configuration for Kindling.","ref":"Kindling.Config.html"},{"type":"function","title":"Kindling.Config.root_resources/0","doc":"Returns the list of configured root resources.","ref":"Kindling.Config.html#root_resources/0"},{"type":"module","title":"Kindling.Converter","doc":"Convert between JSON-style maps and Elixir structs, using the generated resource Ecto schemas as\na guide.","ref":"Kindling.Converter.html"},{"type":"function","title":"Kindling.Converter.convert/2","doc":"Convert a JSON-style map with string keys to a FHIR resource schema. `version_namespace` is\nthe module namespace where your resource schema module have been generated (e.g. `FHIR.R4`)\nand `resource_json` is the map of data.","ref":"Kindling.Converter.html#convert/2"},{"type":"module","title":"Kindling.Converter.DateTime","doc":"Handles dates and datetimes in FHIR's format. Dates may be in ISO8601 format, or may be in a truncated\nform that indicates only certain parts of a date (e.g. year, or year+month, etc). DateTimes work\nsimilarly.","ref":"Kindling.Converter.DateTime.html"},{"type":"function","title":"Kindling.Converter.DateTime.parse/1","doc":"Parse a string containing a FHIR-style datetime. Return `{:ok, DateTime.t(), integer()}` or a parse\nerror.","ref":"Kindling.Converter.DateTime.html#parse/1"},{"type":"function","title":"Kindling.Converter.DateTime.parse!/1","doc":"","ref":"Kindling.Converter.DateTime.html#parse!/1"},{"type":"module","title":"Kindling.Schema","doc":"Tools for working with the JSON schema for FHIR.","ref":"Kindling.Schema.html"},{"type":"function","title":"Kindling.Schema.add_choice_data/2","doc":"","ref":"Kindling.Schema.html#add_choice_data/2"},{"type":"function","title":"Kindling.Schema.all_resources/1","doc":"","ref":"Kindling.Schema.html#all_resources/1"},{"type":"function","title":"Kindling.Schema.get_choice_data/1","doc":"","ref":"Kindling.Schema.html#get_choice_data/1"},{"type":"function","title":"Kindling.Schema.refs_recursive/2","doc":"Given a schema map in the style returned by `&schema_map/1`, and a root resource name (e.g.\n`\"Encounter\"`), return a `MapSet` of resources that are referenced (recursively) from the\nresource. This can be used to determine the set of resource schemas that need to be generated\nto full support that resource as a fully-defined set or schemas.","ref":"Kindling.Schema.html#refs_recursive/2"},{"type":"function","title":"Kindling.Schema.schema_map/1","doc":"Load and process a JSON schema file for the given `version`, return the map defining the spec\nfor that version.","ref":"Kindling.Schema.html#schema_map/1"},{"type":"type","title":"Kindling.Schema.version_string/0","doc":"A string representing a particular FHIR version (\"R5\" | \"R4B\" | \"R4\" | \"R3\" ).","ref":"Kindling.Schema.html#t:version_string/0"},{"type":"module","title":"Kindling.Schema.Resource","doc":"Functions for working on entries of the `definitions` section of the FHIR schema, which define\nthe Resources available in FHIR.","ref":"Kindling.Schema.Resource.html"},{"type":"function","title":"Kindling.Schema.Resource.all_fields/1","doc":"Given the definition `df` return a list of all the properties which are \"simple values\", i.e.\narrays, enum values, or base value types like integer or string.","ref":"Kindling.Schema.Resource.html#all_fields/1"},{"type":"function","title":"Kindling.Schema.Resource.class_name/1","doc":"Convert from a FHIR spec resource type name (\"Encounter_patient\") to the corresponding\nElixir-style module name as a string (\"Encounter.Patient\").","ref":"Kindling.Schema.Resource.html#class_name/1"},{"type":"function","title":"Kindling.Schema.Resource.class_name?/1","doc":"Return true if the resource type name matches the convention for resource type names (i.e. they\nstart with a capital letter).","ref":"Kindling.Schema.Resource.html#class_name?/1"},{"type":"function","title":"Kindling.Schema.Resource.grouped_properties/2","doc":"Given the definition `df` and a list of root resource types, return a struct which contains\nall the properties of `df`, grouped by the type of field that should be used to represent them (\n:array, :const, :embeds_one, :embeds_many, :has_one, :has_many, :enum, :value).","ref":"Kindling.Schema.Resource.html#grouped_properties/2"},{"type":"function","title":"Kindling.Schema.Resource.properties/1","doc":"Return the property fields of a given definition map.","ref":"Kindling.Schema.Resource.html#properties/1"},{"type":"function","title":"Kindling.Schema.Resource.ref_to_class_name/1","doc":"Convert from a definition reference in the FHIR spec (i.e. a string like\n`\"#/definitions/ResourceName\"`) to the corresponding Elixir-style module name (i.e.\n`ResourceName`) as a string.","ref":"Kindling.Schema.Resource.html#ref_to_class_name/1"},{"type":"function","title":"Kindling.Schema.Resource.refs/1","doc":"Return all properties of `df` that are references to other resource (i.e. are not simple value\ntypes).","ref":"Kindling.Schema.Resource.html#refs/1"},{"type":"function","title":"Kindling.Schema.Resource.required/1","doc":"Return a map of all properties (simple or reference) that are required under the FHIR spec.","ref":"Kindling.Schema.Resource.html#required/1"},{"type":"function","title":"Kindling.Schema.Resource.required?/2","doc":"Return true if `key` is required in the definition `df`, else return false.","ref":"Kindling.Schema.Resource.html#required?/2"},{"type":"function","title":"Kindling.Schema.Resource.required_fields/1","doc":"Given the definition `df` return a list of all the properties which are \"simple values\", i.e.\narrays, enum values, or base value types like integer or string AND are required by the FHIR\nspec.","ref":"Kindling.Schema.Resource.html#required_fields/1"},{"type":"type","title":"Kindling.Schema.Resource.definition/0","doc":"A string-keyed map that represents one resource in the `definitions` section of the FHIR schema.","ref":"Kindling.Schema.Resource.html#t:definition/0"},{"type":"type","title":"Kindling.Schema.Resource.grouped_properties_struct/0","doc":"","ref":"Kindling.Schema.Resource.html#t:grouped_properties_struct/0"},{"type":"type","title":"Kindling.Schema.Resource.schema/0","doc":"An `Ecto.Schema` struct representing a FHIR Resource.","ref":"Kindling.Schema.Resource.html#t:schema/0"},{"type":"type","title":"Kindling.Schema.Resource.t/0","doc":"A module representing a given FHIR resource, generated by\n[mix kindling.generate_schemas](`Mix.Tasks.Kindling.GenerateSchemas`).","ref":"Kindling.Schema.Resource.html#t:t/0"},{"type":"module","title":"Kindling.SchemaDownloader","doc":"Handles downloading and unzipping FHIR JSON schemas from the hl7 server.","ref":"Kindling.SchemaDownloader.html"},{"type":"function","title":"Kindling.SchemaDownloader.download_choices/1","doc":"","ref":"Kindling.SchemaDownloader.html#download_choices/1"},{"type":"function","title":"Kindling.SchemaDownloader.download_version/1","doc":"Download and unzip the JSON schema file for the given `version` into the _build directory for\nthe library to reference.","ref":"Kindling.SchemaDownloader.html#download_version/1"},{"type":"function","title":"Kindling.SchemaDownloader.ensure_choices!/1","doc":"","ref":"Kindling.SchemaDownloader.html#ensure_choices!/1"},{"type":"function","title":"Kindling.SchemaDownloader.ensure_version!/1","doc":"Check if the `version` JSON schema is downloaded and unzipped for use. If not, download and unzip\nit. If that download & unzip fails, raise an error.","ref":"Kindling.SchemaDownloader.html#ensure_version!/1"},{"type":"function","title":"Kindling.SchemaDownloader.handle_utf16/1","doc":"","ref":"Kindling.SchemaDownloader.html#handle_utf16/1"},{"type":"module","title":"Kindling.Templates","doc":"Template functions for generating FHIR Resource modules.","ref":"Kindling.Templates.html"},{"type":"function","title":"Kindling.Templates.render/1","doc":"","ref":"Kindling.Templates.html#render/1"},{"type":"function","title":"Kindling.Templates.render_embedded/1","doc":"","ref":"Kindling.Templates.html#render_embedded/1"},{"type":"function","title":"Kindling.Templates.resource_code/5","doc":"","ref":"Kindling.Templates.html#resource_code/5"},{"type":"function","title":"Kindling.Templates.to_source_file_name/1","doc":"","ref":"Kindling.Templates.html#to_source_file_name/1"},{"type":"function","title":"Kindling.Templates.write_code/5","doc":"","ref":"Kindling.Templates.html#write_code/5"},{"type":"module","title":"Kindling.Templates.Functions","doc":"Helper functions used the in generator templates.","ref":"Kindling.Templates.Functions.html"},{"type":"function","title":"Kindling.Templates.Functions.enum_value_string/1","doc":"","ref":"Kindling.Templates.Functions.html#enum_value_string/1"},{"type":"function","title":"Kindling.Templates.Functions.fhir_type_to_ecto/1","doc":"","ref":"Kindling.Templates.Functions.html#fhir_type_to_ecto/1"},{"type":"module","title":"Kindling.Version","doc":"Functions for working with FHIR versions e.g. R3, R4, R5, STU3, etc.","ref":"Kindling.Version.html"},{"type":"function","title":"Kindling.Version.version_dir/1","doc":"","ref":"Kindling.Version.html#version_dir/1"},{"type":"module","title":"Kindling.Xml","doc":"","ref":"Kindling.Xml.html"},{"type":"function","title":"Kindling.Xml.parse/1","doc":"","ref":"Kindling.Xml.html#parse/1"},{"type":"function","title":"Kindling.Xml.process/1","doc":"","ref":"Kindling.Xml.html#process/1"},{"type":"function","title":"Kindling.Xml.reduce_child/2","doc":"","ref":"Kindling.Xml.html#reduce_child/2"},{"type":"task","title":"mix kindling.generate_schemas","doc":"Generates FHIR schemas","ref":"Mix.Tasks.Kindling.GenerateSchemas.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}